"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormsCompositeGeneric = exports.FormGeneric = exports.FieldGeneric = exports.FieldValueGeneric = exports.FieldKeyGeneric = void 0;
const base_1 = require("./base");
const content_1 = require("./content");
const geometry_1 = require("./geometry");
class FieldKeyGeneric extends (0, content_1.WithWords)(base_1.PageHostedApiBlockWrapper) {
    constructor(block, parentField) {
        super(block, parentField.parentPage);
        this._parentField = parentField;
        this._geometry = new geometry_1.Geometry(block.Geometry, this);
    }
    get geometry() {
        return this._geometry;
    }
    get parentField() {
        return this._parentField;
    }
    get confidence() {
        return this._dict.Confidence;
    }
    getOcrConfidence(aggMethod = "MEAN") {
        return (0, base_1.aggregate)(this.listWords().map((w) => w.confidence), aggMethod);
    }
    html({ includeBlockTypes = null, skipBlockTypes = null } = {}) {
        if (skipBlockTypes) {
            skipBlockTypes = (0, base_1.normalizeOptionalSet)(skipBlockTypes);
            if (skipBlockTypes.has("KEY"))
                return "";
        }
        return (0, base_1.escapeHtml)(this.getText({ includeBlockTypes, skipBlockTypes }));
    }
    str() {
        return this.text;
    }
}
exports.FieldKeyGeneric = FieldKeyGeneric;
class FieldValueGeneric extends (0, content_1.buildWithContent)()(base_1.PageHostedApiBlockWrapper) {
    constructor(valueBlock, parentField) {
        super(valueBlock, parentField.parentPage);
        this._parentField = parentField;
        this._geometry = new geometry_1.Geometry(valueBlock.Geometry, this);
    }
    get confidence() {
        return this._dict.Confidence;
    }
    get geometry() {
        return this._geometry;
    }
    get isCheckbox() {
        return this.listContent({ includeBlockTypes: ["SELECTION_ELEMENT"] }).length == 1;
    }
    get isSelected() {
        const selEls = this.listContent({
            includeBlockTypes: ["SELECTION_ELEMENT"],
        });
        return selEls.length === 1 ? selEls[0].selectionStatus === "SELECTED" : null;
    }
    get parentField() {
        return this._parentField;
    }
    get selectionStatus() {
        const selEls = this.listContent({
            includeBlockTypes: ["SELECTION_ELEMENT"],
        });
        return selEls.length === 1 ? selEls[0].selectionStatus : null;
    }
    getOcrConfidence(aggMethod = "MEAN") {
        return (0, base_1.aggregate)(this.listContent().map((c) => c.confidence), aggMethod);
    }
    html({ includeBlockTypes = null, skipBlockTypes = null } = {}) {
        if (skipBlockTypes) {
            skipBlockTypes = (0, base_1.normalizeOptionalSet)(skipBlockTypes);
            if (skipBlockTypes.has("VALUE"))
                return "";
        }
        return (0, base_1.escapeHtml)(this.getText({ includeBlockTypes, skipBlockTypes }));
    }
    str() {
        return this.text;
    }
}
exports.FieldValueGeneric = FieldValueGeneric;
class FieldGeneric {
    constructor(keyBlock, parentForm) {
        this._parentForm = parentForm;
        this._value = null;
        this._key = new FieldKeyGeneric(keyBlock, this);
        let valueBlockIds = [];
        (keyBlock.Relationships || []).forEach((rs) => {
            if (rs.Type == "VALUE") {
                valueBlockIds = valueBlockIds.concat(rs.Ids);
            }
        });
        if (valueBlockIds.length > 1) {
            const fieldLogName = this._key ? `field '${this._key.text}'` : "unnamed form field";
            console.warn(`Got ${valueBlockIds.length} value blocks for ${fieldLogName} (Expected 0-1). Including first only.`);
        }
        if (valueBlockIds.length) {
            const valBlockId = valueBlockIds[0];
            const valBlock = parentForm.parentPage.getBlockById(valBlockId);
            if (!valBlock) {
                console.warn(`Document missing child block ${valBlockId} referenced by value for field key ${this.key.id}`);
            }
            else {
                this._value = new FieldValueGeneric(valBlock, this);
            }
        }
    }
    get blockType() {
        return this._key.blockType;
    }
    get confidence() {
        const scores = [];
        if (this._key) {
            scores.push(this._key.confidence || 0);
        }
        if (this._value) {
            scores.push(this._value.confidence || 0);
        }
        if (scores.length) {
            return scores.reduce((acc, next) => acc + next, 0) / scores.length;
        }
        else {
            return 0;
        }
    }
    get childBlockIds() {
        return this._key.childBlockIds;
    }
    get dict() {
        return this._key.dict;
    }
    get id() {
        return this._key.id;
    }
    get isSelected() {
        return this.value ? this.value.isSelected : null;
    }
    get isCheckbox() {
        var _a;
        return !!((_a = this.value) === null || _a === void 0 ? void 0 : _a.isCheckbox);
    }
    get key() {
        return this._key;
    }
    get parentForm() {
        return this._parentForm;
    }
    get parentPage() {
        return this._parentForm.parentPage;
    }
    get selectionStatus() {
        return this.value ? this.value.selectionStatus : null;
    }
    get text() {
        var _a;
        return `${this._key.text}: ${((_a = this._value) === null || _a === void 0 ? void 0 : _a.text) || ""}`;
    }
    get value() {
        return this._value;
    }
    getOcrConfidence(aggMethod = "MEAN") {
        const keyValContent = (this._value ? this._value.listContent() : []).concat(this._key.listWords());
        return (0, base_1.aggregate)(keyValContent.map((c) => c.confidence), aggMethod);
    }
    relatedBlockIdsByRelType(relType) {
        return this._key.relatedBlockIdsByRelType(relType);
    }
    html(opts) {
        let renderKey = (0, base_1.doesFilterAllowBlockType)(opts, this.key.blockType);
        let renderValue = this.value && (0, base_1.doesFilterAllowBlockType)(opts, this.value.blockType);
        if (opts && opts.skipBlockTypes) {
            const skipSpec = (0, base_1.normalizeOptionalSet)(opts.skipBlockTypes);
            if (skipSpec.has("KEY"))
                renderKey = false;
            if (skipSpec.has("VALUE"))
                renderValue = false;
        }
        if (!renderKey && !renderValue)
            return "";
        const keyPartial = renderKey ? ` label="${(0, base_1.escapeHtml)(this._key.getText(opts), { forAttr: true })}"` : "";
        const valPartial = renderValue
            ? ` value="${(0, base_1.escapeHtml)(this.value ? this.value.getText(opts) : "", { forAttr: true })}"`
            : "";
        return `<input${keyPartial} type="text" disabled${valPartial} />`;
    }
    str() {
        return `\nField\n==========\nKey: ${this._key ? this._key.str() : ""}\nValue: ${this._value ? this._value.str() : ""}`;
    }
}
exports.FieldGeneric = FieldGeneric;
class FormGeneric {
    constructor(keyBlocks, parentPage) {
        this._fields = [];
        this._fieldsMap = {};
        this._parentPage = parentPage;
        keyBlocks.forEach((keyBlock) => {
            const f = new FieldGeneric(keyBlock, this);
            this._fields.push(f);
            const fieldKeyText = f.key.text || "";
            if (fieldKeyText) {
                if (fieldKeyText in this._fieldsMap) {
                    if (f.confidence > this._fieldsMap[fieldKeyText].confidence) {
                        this._fieldsMap[fieldKeyText] = f;
                    }
                }
                else {
                    this._fieldsMap[fieldKeyText] = f;
                }
            }
        });
    }
    get nFields() {
        return this._fields.length;
    }
    get parentPage() {
        return this._parentPage;
    }
    get text() {
        return this.listFields()
            .map((f) => f.text)
            .join("\n");
    }
    getFieldByKey(key) {
        return this._fieldsMap[key] || null;
    }
    html(opts = {}) {
        const fieldHtmls = this.listFields()
            .map((f) => f.html(opts))
            .filter((s) => s);
        return fieldHtmls.length ? `<form>\n${(0, base_1.indent)(fieldHtmls.join("\n"))}\n</form>` : "<form></form>";
    }
    iterFields(skipFieldsWithoutKey = false) {
        return (0, base_1.getIterable)(() => this.listFields(skipFieldsWithoutKey));
    }
    listFields(skipFieldsWithoutKey = false) {
        return skipFieldsWithoutKey ? this._fields.filter((f) => f.key) : this._fields.slice();
    }
    searchFieldsByKey(key) {
        const searchKey = key.toLowerCase();
        return this._fields.filter((field) => field.key && field.key.text.toLowerCase().indexOf(searchKey) >= 0);
    }
    str() {
        return this._fields.map((f) => f.str()).join("\n");
    }
}
exports.FormGeneric = FormGeneric;
class FormsCompositeGeneric {
    constructor(forms, parentDocument) {
        this._forms = forms;
        this._parentDocument = parentDocument;
    }
    get nFields() {
        return this._forms.reduce((acc, next) => acc + next.nFields, 0);
    }
    get parentDocument() {
        return this._parentDocument;
    }
    get text() {
        return this._forms.map((f) => f.text).join("\n\n");
    }
    getFieldByKey(key) {
        for (const form of this._forms) {
            const result = form.getFieldByKey(key);
            if (result)
                return result;
        }
        return null;
    }
    html() {
        const pageHtmls = this._forms.map((form, ixForm) => `<div class="form-page" id="form-page-${ixForm}">\n${(0, base_1.indent)(form
            .listFields()
            .map((f) => `${f.html()}\n`)
            .join(""))}</div>\n`);
        return `<form>\n${(0, base_1.indent)(pageHtmls.join(""))}</form>`;
    }
    iterFields(skipFieldsWithoutKey = false) {
        return (0, base_1.getIterable)(() => this.listFields(skipFieldsWithoutKey));
    }
    listFields(skipFieldsWithoutKey = false) {
        const allFields = [].concat(...this._forms.map((form) => form.listFields()));
        if (skipFieldsWithoutKey) {
            return allFields.filter((f) => f.key);
        }
        else {
            return allFields;
        }
    }
    searchFieldsByKey(key) {
        return [].concat(...this._forms.map((f) => f.searchFieldsByKey(key)));
    }
    str() {
        return this._forms.map((f) => f.str()).join("\n");
    }
}
exports.FormsCompositeGeneric = FormsCompositeGeneric;
//# sourceMappingURL=form.js.map